from django.conf import settings
from axes.models import AccessAttempt
import axes
import logging

# see if the user has overridden the failure limit
if hasattr(settings, 'LOGIN_FAILURE_LIMIT'):
    FAILURE_LIMIT = settings.LOGIN_FAILURE_LIMIT
else:
    FAILURE_LIMIT = 3

# see if the user has overridden the failure reset setting
if hasattr(settings, 'LOGIN_FAILURE_RESET'):
    FAILURE_RESET = settings.LOGIN_FAILURE_RESET
else:
    FAILURE_RESET = True

def query2str(items):
    return '\n'.join(['%s=%s' % (k, v) for k,v in items])

log = logging.getLogger('axes.watch_login')
log.info('BEGIN LOG')
log.info('Using django-axes ' + axes.get_version())

def watch_login(func, failures):
    """
    Used to decorate the django.contrib.admin.site.login method.
    """

    def decorated_login(request, *args, **kwargs):
        # share some useful information
        if func.__name__ != 'decorated_login':
            log.info('Calling decorated function: %s' % func)
            if args: log.info('args: %s' % args)
            if kwargs: log.info('kwargs: %s' % kwargs)

        # call the login function
        response = func(request, *args, **kwargs)

        if func.__name__ == 'decorated_login':
            # if we're dealing with this function itself, don't bother checking
            # for invalid login attempts.  I suppose there's a bunch of
            # recursion going on here that used to cause one failed login
            # attempt to generate 10+ failed access attempt records (with 3
            # failed attempts each supposedly)
            return response

        # only check when there's been an HTTP POST
        if request.method == 'POST':
            # see if the login was successful
            if response and not response.has_header('location') and response.status_code != 302:
                log.debug('Failure dict (begin): %s' % failures)
                ip = request.META.get('REMOTE_ADDR', '')
                ua = request.META.get('HTTP_USER_AGENT', '<unknown>')

                key = '%s:%s' % (ip, ua)

                # make sure we have an item for this key
                try:
                    failures[key]
                    log.debug('Key %s exists' % key)
                except KeyError:
                    log.debug('Creating key %s' % key)
                    failures[key] = 0

                # add a failed attempt for this user
                failures[key] += 1

                log.info('Adding a failure for %s; %i failure(s)' % (key, failures[key]))
                #log.debug('Request: %s' % request)

                # if we reach or surpass the failure limit, create an
                # AccessAttempt record
                if failures[key] >= FAILURE_LIMIT:
                    log.info('=================================')
                    log.info('Creating access attempt record...')
                    log.info('=================================')
                    attempt = AccessAttempt.objects.create(
                        user_agent=ua,
                        ip_address=ip,
                        get_data=query2str(request.GET.items()),
                        post_data=query2str(request.POST.items()),
                        http_accept=request.META.get('HTTP_ACCEPT', '<unknown>'),
                        path_info=request.META.get('PATH_INFO', '<unknown>'),
                        failures_since_start=failures[key]
                    )

                    if FAILURE_RESET:
                        del(failures[key])

                log.debug('Failure dict (end): %s' % failures)
                log.info('-' * 79)

        return response
    return decorated_login


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup
import axes
import sys, os

def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
axes_dir = 'axes'

for path, dirs, files in os.walk(axes_dir):
    # ignore hidden directories and files
    for i, d in enumerate(dirs):
        if d.startswith('.'): del dirs[i]

    if '__init__.py' in files:
        packages.append('.'.join(fullsplit(path)))
    elif files:
        data_files.append((path, [os.path.join(path, f) for f in files]))

setup(
    name='django-axes',
    version=axes.get_version(),
    url='http://code.google.com/p/django-axes/',
    author='Josh VanderLinden',
    author_email='codekoala@gmail.com',
    license='MIT',
    packages=packages,
    data_files=data_files,
    description="Keep track of failed login attempts in Django-powered sites.",
    long_description="""
django-axes is a very simple way for you to keep track of failed login attempts, both for the Django admin and for the rest of your site.  All you need to do is install the application, a middleware, and syncdb!
""",
    keywords='django, security, authentication',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: Log Analysis',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
        'Topic :: Security',
        'Topic :: System :: Logging',
    ]
)
